clear
cd "N:\reallocation\Data"


// INPUT 
// 1) BLS_employment_industry_monthly.xlsx
// 2) bls_naics.xlsx
// 3) BEA_NAICS_BRIDGE.xlsx

// OUTPUT 
// 1) bea66_employment_bls_quarterly.dta




import excel "BLS_employment_industry_monthly.xlsx", sheet("Labels") clear firstrow
save temp_bls_employment_code_labels.dta, replace

import excel "BLS_employment_industry_monthly.xlsx", sheet("Data") firstrow clear

gen month = ym(1995, 1) + _n - 1
format month %tm 
gen year = 1960 + floor(month/12)
gen quarter = qofd(dofm(month))
format %tq quarter
rename SeriesID Date

*collapse (firstnm) event ///
*(mean) GPR* LGPR* N* MCOMP* FREQ_* RESID*, by(quarter)

preserve
drop Date month year quarter
describe, replace clear
list
rename position sector
save temp_bls_employment_simple_labels.dta, replace
restore




global i = 1
foreach var of varlist CES0000000001-CES9093200001 {
di `i'
destring `var', replace
format `var' %10.0g
rename `var' vv$i
global i = $i+1
}



reshape long vv, i(Date) j(sector)
rename vv emp

merge m:1 sector using "temp_bls_employment_simple_labels.dta"
drop _merge format name type isnumer format vallab
rename varlab Industrycode




merge m:1 Industrycode using "temp_bls_employment_code_labels.dta"
rename Industrycode industrycode
rename sector sectornum
rename Industryname industryname
drop _merge

sort sectornum quarter
drop month
drop year
drop Date

label var sectornum "Sector Number BLS Emp.Situation"
label var quarter "quarter"
label var emp "Total Number of Employees"

collapse (firstnm) industryname sectornum ///
(mean) emp, by(industrycode quarter)

label var sectornum "Sector Number BLS Emp.Situation"
label var quarter "quarter"
label var emp "Total Number of Employees"

label var industryname "BLS Industry Name"
rename industryname BLS_industryname

gen BLS_code = substr(industrycode ,4,8)

xtset sectornum quarter
//browse sectornum BLS_code if tin(2019q1,2019q1) 




keep emp industrycode BLS_code BLS_industryname quarter
save temp_crosswalk_part0, replace






import excel BLS_NAICS_BRIDGE.xlsx, sheet("Sheet1") firstrow allstring clear

//browse industry_code naics_code if strpos(naics_code,",")>0

********************************
**** FIX 1 *********************
********************************
* Replace sector number for automobile industry
replace naics_code = "3361" if naics_code=="3361,2,3"


drop if strpos(naics_code,",")>0
drop if strpos(naics_code,"part")>0
drop if strpos(naics_code,"-")>0

gen naics_cut = substr(naics_code ,1,5)
keep industry_code naics_code industry_name naics_cut
sort naics_cut
rename industry_code BLS_code
rename industry_name NAICS_name

gen length = strlen(naics_cut)
gen naics5 = naics_cut if length==5
gen naics4 = naics_cut if length==4
gen naics3 = naics_cut if length==3
gen naics2 = naics_cut if length==2
gen naics = naics_cut 

order  naics_code naics_cut BLS_code NAICS_name
save temp_crosswalk_part1, replace

order BLS_code naics naics2 naics3 naics4 naics5
* browse BLS_code naics naics2 naics3 naics4 naics5

keep BLS_code naics naics2 naics3 naics4 naics5
save temp_crosswalk_part1_simple, replace




import excel BEA_NAICS_BRIDGE.xlsx, sheet("BEA") firstrow allstring clear
drop D-K

sencode BEAName, gen(BEAnumber_) gsort(BEAName)
compress 
gen BEA66 = BEAnumber_
sort BEA66
drop BEAnumber_
gen m = _n
by BEA66 : gen dm = m - m[_n-1]

gen length = strlen(NAICSCodes)
gen naics4 = NAICSCodes if length==4
gen naics3 = NAICSCodes if length==3
gen naics2 = NAICSCodes if length==2
gen naics = NAICSCodes 

gen original_naics = naics



********************************
**** FIX 2 *********************
********************************
* Missing employment in subsector of financial industry
replace naics = "52" if naics=="525"
replace naics = "336" if naics=="3364"
replace naics = "52" if naics=="523"

* Assign to forestry and fishing the employment of logging which is technically a subsector in NAICS
replace naics = "1133" if naics=="113" 



order naics naics2 naics3 naics4 BEA66
sort naics
*browse  naics naics2 naics3 naics4 BEA66 

keep naics naics2 naics3 naics4 BEA66 BEAName BEACode
save temp_crosswalk_part2_simple, replace









* CROSSWALK0
* All 175 BLS industries, by name, employees, and 8-digit BLS_code

* CROSSWALK1
* For each of 837 8-digit BLS Industries, we have a unique NAICS code, either 2 or 3 or 4 digits

* CROSSWALK2
* For each of 99 NAICS 2 3 or 4 digits, we have a unique BEA66 code 


* Step 1 : Merge 175 BLS industries with BLS-NAICS industries
use temp_crosswalk_part0, clear

merge m:1 BLS_code using temp_crosswalk_part1_simple.dta, generate(merge0)



* Step 2: Merge with BEA
joinby naics using temp_crosswalk_part2_simple.dta, _merge(merge1)




* Step 3: Save employment data for BEA
sort quarter BEA66
drop if BEA66==.

keep if tin(2005q1,2022q4)

collapse emp (firstnm) BEACode, by(BEA66 quarter)

erase temp_bls_employment_code_labels.dta
erase temp_bls_employment_simple_labels.dta
erase temp_crosswalk_part0.dta
erase temp_crosswalk_part1.dta
erase temp_crosswalk_part1_simple.dta
erase temp_crosswalk_part2_simple.dta


save tfile_bea66_employment_quarterly, replace








