* Import productivity data from BLS website
* and converts them into BEA-66 productivity data
* https://www.bea.gov/industry/industry-underlying-estimates

import excel "N:\reallocation\Data\major-industry-total-factor-productivity-klems.xlsx", sheet("MachineReadable") firstrow clear

keep if ///
(Measure=="Capital input" & Units=="Index (2012=100)") | ///
(Measure=="Combined inputs" & Units=="Index (2012=100)") | ///
(Measure=="Intermediate inputs" & Units=="Index (2012=100)") | ///
(Measure=="Labor input" & Units=="Index (2012=100)") | ///
(Measure=="Real sectoral output" & Units=="Index (2012=100)") | ///
(Measure=="Sectoral output" & Units=="Billions of current dollars") | ///
(Measure=="Total factor productivity" & Units=="Index (2012=100)")


destring Year, replace

destring Value, force replace

keep NAICS Year Value Measure Industry

sort Year NAICS

save temp_prod_naics, replace

use temp_prod_naics, clear

encode Measure, gen(MeasureI)
drop Measure

reshape wide Value, i(NAICS Year) j(MeasureI) 

rename Value1 BLS_K
rename Value2 BLS_CI
rename Value3 BLS_II
rename Value4 BLS_L
rename Value5 BLS_RSO
rename Value6 BLS_SO
rename Value7 BLS_TFP

label var BLS_K  "BLS Capital Input - Index (2012=100)"
label var BLS_CI "BLS Combined Inputs - Index (2012=100)"
label var BLS_II "BLS Intermediate Inputs - Index (2012=100)"
label var BLS_L  "BLS Labor - Index (2012=100)"
label var BLS_RSO "BLS Real sectoral output - Index (2012=100)"
label var BLS_SO  "BLS Sectoral output - Billions of current dollars"
label var BLS_TFP "BLS TFP - Index (2012=100)"

order NAICS Industry Year BLS_*


gen BEACode = NAICS

tabulate NAICS


* Drop superindustries for which there are better sub-classifications
drop if NAICS == "11"
drop if NAICS == "21"
drop if NAICS == "42,44-45"
drop if NAICS == "48-49"
drop if NAICS == "51"
drop if NAICS == "52"
drop if NAICS == "52-53"
drop if NAICS == "53"
drop if NAICS == "54"
drop if NAICS == "54-56"
drop if NAICS == "54-81"
drop if NAICS == "56"
drop if NAICS == "61-62"
drop if NAICS == "62"
drop if NAICS == "71"
drop if NAICS == "71-72"
drop if NAICS == "72"
drop if NAICS == "DM"
drop if NAICS == "MN"
drop if NAICS == "ND"


replace BEACode = "111CA" if NAICS=="111-112"
replace BEACode = "113FF" if NAICS=="113-115"
replace BEACode = "311FT" if NAICS=="311-312"
replace BEACode = "313TT" if NAICS=="313-314"
replace BEACode = "315AL" if NAICS=="315-316"
replace BEACode = "3361MV" if NAICS=="3361-3363"
replace BEACode = "3364OT" if NAICS=="3364-3369"
replace BEACode = "487OS" if NAICS=="487,488,492"
replace BEACode = "513" if NAICS=="515,517"
replace BEACode = "514" if NAICS=="518-519"
replace BEACode = "521CI" if NAICS=="521-522"
replace BEACode = "532RL" if NAICS=="532-533"
replace BEACode = "5412OP" if NAICS=="5412-5414,5416-5419"
replace BEACode = "711AS" if NAICS=="711-712"


* Appoximation one NAICS to MANY BEACode

* NEED TO EXPAND OBSERVATIONS WHEN THERE ARE MORE BEA FOR EACH NAICS

* Correction 1: 4 BEA for 1 NAICS
expand 2 if NAICS=="44,45", gen(dupindicator1)
expand 2 if NAICS=="44,45" & dupindicator1==0, gen(dupindicator2)
expand 2 if NAICS=="44,45" & dupindicator1==0 & dupindicator2==0, gen(dupindicator3)

replace BEACode = "4A0" if NAICS=="44,45" 
replace BEACode = "445" if NAICS=="44,45" & dupindicator1==1
replace BEACode = "452" if NAICS=="44,45" & dupindicator2==1
replace BEACode = "441" if NAICS=="44,45" & dupindicator3==1


* Correction 2: 2 BEA for 1 NAICS
expand 2 if NAICS=="531", gen(dupindicatorX)
replace BEACode = "HS" if NAICS=="531" 
replace BEACode = "ORE" if NAICS=="531" & dupindicatorX==1


* Correction 2: 2 BEA for 1 NAICS
expand 2 if NAICS=="622-623", gen(dupindicatorY)
replace BEACode = "622" if NAICS=="622-623" 
replace BEACode = "623" if NAICS=="622-623" & dupindicatorY==1


tabulate BEACode
drop dupindicator*


erase temp_prod_naics.dta



* Save data for later merge with productivity from BEA NIPA
save tfile_productivity_naics_bea66.dta, replace




