//% N sector NK model with Input-Output Linkages

// Include blocks defined elsewhere
@#include "definition_block_nsec.mod" 
@#include "definition_block_io.mod" 
@#include "definition_block_lab.mod" 

// Define variables
var chi Lab_costs Price_costs w VA C Ctot Ctotg Ctots M C_g C_s p_g p_s N pi pi_g pi_s r om_g om_s vi Y Yagg
Y_g Y_s L_g L_s M_g M_s MC_g MC_s PL_s PL_g
    @#for i in 1:nsec
        P_@{i}
    @#endfor
    @#for i in 1:nsec
        L_@{i}
    @#endfor
    @#for i in 1:nsec
        M_@{i}
    @#endfor
    @#for i in 1:nsec
        Y_@{i}
    @#endfor
    @#for i in 1:nsec
        PM_@{i}
    @#endfor
    @#for i in 1:nsec
        PL_@{i}
    @#endfor
    @#for i in 1:nsec
        MC_@{i}
    @#endfor
    @#for i in 1:nsec
        A_@{i}
    @#endfor
    ;

varexo eps_om eps_i epschi
    @#for i in 1:nsec
        epsA_@{i}
    @#endfor
;

parameters gamma psi beta phi epsilon rho rho_om1 rho_tfp1 rho_om2 rho_tfp2 ombar
rhoi rhoirule ilabcosts

    sigma_i
    sigma_om
    sigma_L_agg
    isigma_tfp
    
    @#for i in 1:nsec
        gammag_@{i}
    @#endfor
    @#for i in 1:nsec
        alpha_@{i}
    @#endfor
    @#for i in 1:nsec
        gammas_@{i}
    @#endfor
    @#for i in 1:nsec
        @#for j in 1:nsec
            beta_@{i}_@{j}
        @#endfor
    @#endfor
    @#for i in 1:nsec
        epsY_@{i}
    @#endfor
    @#for i in 1:nsec
        epsM_@{i}
    @#endfor
    @#for i in 1:nsec
        kappa_@{i}
    @#endfor
    @#for i in 1:nsec
        dummyg_@{i}
    @#endfor
    @#for i in 1:nsec
        dummys_@{i}
    @#endfor
    @#for i in 1:nsec
        cl_@{i}
    @#endfor
    @#for i in 1:nsec
        clneg_@{i}
    @#endfor
    @#for i in 1:nsec
        cm_@{i}
    @#endfor
    ;  
      
load params_val.mat;

sigma_i = sigma_i_val;
sigma_L_agg = sigma_L_agg_val;
sigma_om = sigma_om_val;
isigma_tfp = isigma_tfp_val;
ilabcosts = ilabcosts_val;

gamma = 2;
psi = 1;
beta = beta_val;
phi = phi_val;
epsilon = 10;
rho = rho_val;
rho_om1 = rho_om1_val;
rho_om2 = rho_om2_val;
rho_tfp1 = rho_tfp1_val;
rho_tfp2 = rho_tfp2_val;
rhoi = rhoi_val;
rhoirule = rhoirule_val;
ombar = ombar_val;

@#for i in 1:nsec
    gammag_@{i} = modgammag(@{i});
@#endfor
@#for i in 1:nsec
    gammas_@{i} = modgammas(@{i});
@#endfor
@#for i in 1:nsec
    alpha_@{i} = modalpha(@{i});
@#endfor
@#for i in 1:nsec
    @#for j in 1:nsec
        beta_@{i}_@{j} = modbeta(@{i},@{j});
    @#endfor
@#endfor
@#for i in 1:nsec
    epsY_@{i} = modepsY(@{i});
@#endfor
@#for i in 1:nsec
    epsM_@{i} = modepsM(@{i});
@#endfor
@#for i in 1:nsec
    kappa_@{i} = modkappa(@{i});
@#endfor
@#for i in 1:nsec
    dummyg_@{i} = goods(@{i});
@#endfor
@#for i in 1:nsec
    dummys_@{i} = services(@{i});
@#endfor
@#for i in 1:nsec
    cl_@{i}=modcl(@{i});
@#endfor
@#for i in 1:nsec
    clneg_@{i}=modclneg(@{i});
@#endfor
@#for i in 1:nsec
    cm_@{i}=modcm(@{i});
@#endfor

model(bytecode);

//% Relative Price of Goods
exp(p_g) = (1
@#for j in 1:nsec
    *(exp(P_@{j}))^gammag_@{j}
@#endfor
);

//% Relative Price of Services
exp(p_s) = (1
@#for j in 1:nsec
    *(exp(P_@{j}))^gammas_@{j}
@#endfor
);

//% Overall Consumption Basket
exp(C) = (exp(C_g)/exp(om_g))^exp(om_g)*(exp(C_s)/exp(om_s))^exp(om_s) ;

//% Household Euler Equation
exp(C)^(-gamma) = beta*(exp(C(+1))^-gamma)*exp(r)/exp(pi(+1));

//% Relative Inflation: Goods vs Services
exp(pi_g)*exp(C_g)/exp(C_g(-1))*(exp(om_s))/(exp(om_s(-1))) = exp(pi_s)*exp(C_s)/exp(C_s(-1))*(exp(om_g))/(exp(om_g(-1)));

//% Labor Market Clearing
exp(N) = (
    @#for i in 1:nsec
        +exp(L_@{i}) 
        + ilabcosts*cl_@{i}/2*exp(L_@{i})*(L_@{i} > L_@{i}(-1))*(exp(L_@{i})/exp(L_@{i}(-1))-1)^2
        + ilabcosts*clneg_@{i}/2*exp(L_@{i})*(L_@{i} < L_@{i}(-1))*(exp(L_@{i})/exp(L_@{i}(-1))-1)^2
    @#endfor
    );

Lab_costs = (
    @#for i in 1:nsec
        + ilabcosts*cl_@{i}/2*exp(L_@{i})*(L_@{i} > L_@{i}(-1))*(exp(L_@{i})/exp(L_@{i}(-1))-1)^2
        + ilabcosts*clneg_@{i}/2*exp(L_@{i})*(L_@{i} < L_@{i}(-1))*(exp(L_@{i})/exp(L_@{i}(-1))-1)^2
    @#endfor
    );
    
//% Labor leisure condition
exp(C)^(-gamma)*exp(w) = chi*exp(N)^psi;

//% Relative Price of Goods
exp(p_g) = exp(om_g)*exp(C)/exp(C_g);

//% Relative Price of Services
exp(p_s) = exp(om_s)*exp(C)/exp(C_s);

//% Relative Inflation: Goods vs Overall
exp(pi_g)*exp(C_g)/exp(C_g(-1)) = exp(om_g)/exp(om_g(-1))*(exp(pi))*exp(C)/exp(C(-1));

//% Gross Output
exp(Y) = (
    @#for i in 1:nsec
        +exp(Y_@{i})
    @#endfor
    );

//% Total Consumption (Note: = Value Added)
exp(Ctot) = (
@#for i in 1:nsec
    +gammag_@{i}*exp(C_g)*exp(p_g)/exp(P_@{i}) + gammas_@{i}*exp(C_s)*exp(p_s)/exp(P_@{i})
@#endfor
);

//% Total Consumption: Services
exp(Ctots) = (
@#for i in 1:nsec
    +gammas_@{i}*exp(C_s)*exp(p_s)/exp(P_@{i})
@#endfor
);

//% Total Consumption: Goods
exp(Ctotg) = (
@#for i in 1:nsec
    +gammag_@{i}*exp(C_g)*exp(p_g)/exp(P_@{i}) 
@#endfor
);

@#if io_val
    //% Value Added
    exp(VA) = (
        @#for i in 1:nsec
            +exp(Y_@{i}) - exp(M_@{i})
        @#endfor
    );
@#else
    //% Value Added
    exp(VA) = (
        @#for i in 1:nsec
            +exp(Y_@{i})
        @#endfor
    );
@#endif

//% Total Intermediate Use
exp(M) = (
@#for i in 1:nsec
    @#for j in 1:nsec
        + beta_@{j}_@{i}*(exp(PM_@{j})/exp(P_@{i}))^epsM_@{i}*exp(M_@{j})
    @#endfor
@#endfor
);

//% Total Price Adjustment Costs
Price_costs = (
@#for i in 1:nsec
        + (kappa_@{i}/2)*(exp(pi)*exp(P_@{i})/exp(P_@{i}(-1))-1)^2*exp(Y_@{i})
@#endfor
);


//% Sector-by-Sector Equations:
@#for i in 1:nsec

    //% Intermediates Price Index
    exp(PM_@{i}) = (
    @#for j in 1:nsec
        + beta_@{i}_@{j}*(exp(P_@{j}))^(1-epsM_@{i})
    @#endfor
    )^(1/(1-epsM_@{i})); 

    @#if io_val
    //% Production Function
        exp(Y_@{i}) = exp(A_@{i})*((alpha_@{i})^(1/epsY_@{i})*(exp(M_@{i}))^((epsY_@{i}-1)/epsY_@{i}) 
        + (1-alpha_@{i})^(1/epsY_@{i})*(exp(L_@{i}))^((epsY_@{i}-1)/epsY_@{i}))^(epsY_@{i}/(epsY_@{i}-1));
    //% Intermediates Demand
        exp(MC_@{i})*(alpha_@{i}*exp(Y_@{i})/(exp(M_@{i})))^(1/epsY_@{i}) = exp(PM_@{i});
    //% Labor Demand
        exp(MC_@{i})*((1-alpha_@{i})*exp(Y_@{i})/(exp(L_@{i})))^(1/epsY_@{i}) = exp(PL_@{i});
    //% Market Clearing in Each Sector
        exp(Y_@{i}) = gammag_@{i}*exp(C_g)*exp(p_g)/exp(P_@{i}) + gammas_@{i}*exp(C_s)*exp(p_s)/exp(P_@{i})
        @#for j in 1:nsec
            + beta_@{j}_@{i}*(exp(PM_@{j})/exp(P_@{i}))^epsM_@{i}*exp(M_@{j})
        @#endfor
        ;
    @#else
    //% Production Function
        exp(Y_@{i}) = exp(A_@{i})*exp(L_@{i});
    //% Intermediates Demand
        exp(M_@{i}) = 1;
    //% Labor Demand
        exp(MC_@{i}) = exp(PL_@{i})/exp(A_@{i});
    //% Market Clearing in Each Sector
        exp(Y_@{i}) = gammag_@{i}*exp(C_g)*exp(p_g)/exp(P_@{i}) + gammas_@{i}*exp(C_s)*exp(p_s)/exp(P_@{i});
    @#endif

    //% Labor costs FOC  (allows for firing costs)
    @#if I_hiring_cost
        exp(PL_@{i}) = exp(w) 
        + (L_@{i} > L_@{i}(-1))*(cl_@{i}*exp(w)*exp(L_@{i})*(exp(L_@{i})/exp(L_@{i}(-1))-1)/exp(L_@{i}(-1)) 
        + cl_@{i}/2*exp(w)*(exp(L_@{i})/exp(L_@{i}(-1))-1)^2) 
        + (L_@{i} < L_@{i}(-1))*(clneg_@{i}*exp(w)*exp(L_@{i})*(exp(L_@{i})/exp(L_@{i}(-1))-1)/exp(L_@{i}(-1)) 
        + clneg_@{i}/2*exp(w)*(exp(L_@{i})/exp(L_@{i}(-1))-1)^2) 
        - (L_@{i}(+1) > L_@{i})*beta*((exp(C(+1))/exp(C))^-gamma)*cl_@{i}*exp(w(+1))*(exp(L_@{i}(+1))/exp(L_@{i})-1)*exp(L_@{i}(+1))^2/(exp(L_@{i})^2)
        - (L_@{i}(+1) < L_@{i})*beta*((exp(C(+1))/exp(C))^-gamma)*clneg_@{i}*exp(w(+1))*(exp(L_@{i}(+1))/exp(L_@{i})-1)*exp(L_@{i}(+1))^2/(exp(L_@{i})^2);           
    @#endif 

    //% Rotemberg Pricing FOC
    1 - epsilon + epsilon*exp(MC_@{i})/exp(P_@{i}) 
    - kappa_@{i}*(exp(pi)*exp(P_@{i})/exp(P_@{i}(-1))-1)*exp(pi)*exp(P_@{i})/exp(P_@{i}(-1)) 
    + beta*(exp(C(+1))/exp(C))^(-gamma)*kappa_@{i}*(exp(pi(+1))*exp(P_@{i}(+1))/exp(P_@{i})-1)
    *(exp(pi(+1))*exp(P_@{i}(+1))/exp(P_@{i}))^2/exp(pi(+1))*exp(Y_@{i}(+1))/exp(Y_@{i}) =0; 
    
    @#endfor

    
//% Agg Production Function
exp(Yagg) = ((alpha_1)^(1/epsY_1)*(exp(M))^((epsY_1-1)/epsY_1) + (1-alpha_1)^(1/epsY_1)*(exp(N))^((epsY_1-1)/epsY_1))^(epsY_1/(epsY_1-1));
       
//% Total Output of Goods
exp(Y_g) = (
@#for j in 1:nsec
    +(exp(Y_@{j}))*dummyg_@{j}
@#endfor
);

//% Total Output of Services
exp(Y_s) = (
@#for j in 1:nsec
    +(exp(Y_@{j}))*dummys_@{j}
@#endfor
);

//% Total L of Goods
exp(L_g) = (
@#for j in 1:nsec
    + exp(L_@{j})*dummyg_@{j}
    + ilabcosts*cl_@{i}/2*exp(L_@{i})*(L_@{i} > L_@{i}(-1))*((exp(L_@{i})/exp(L_@{i}(-1))-1)^2)*dummyg_@{j}
    + ilabcosts*clneg_@{i}/2*exp(L_@{i})*(L_@{i} < L_@{i}(-1))*((exp(L_@{i})/exp(L_@{i}(-1))-1)^2)*dummyg_@{j}
@#endfor
);

//% Total L of Services
exp(L_s) = (
@#for j in 1:nsec
    + exp(L_@{j})*dummys_@{j} 
    + ilabcosts*cl_@{i}/2*exp(L_@{i})*(L_@{i} > L_@{i}(-1))*((exp(L_@{i})/exp(L_@{i}(-1))-1)^2)*dummys_@{j}
    + ilabcosts*clneg_@{i}/2*exp(L_@{i})*(L_@{i} < L_@{i}(-1))*((exp(L_@{i})/exp(L_@{i}(-1))-1)^2)*dummys_@{j}
@#endfor
);

//% Total M of Goods
exp(M_g) = (
@#for j in 1:nsec
    +(exp(M_@{j}))*dummyg_@{j}
@#endfor
);

//% Total M of Services
exp(M_s) = (
@#for j in 1:nsec
    +(exp(M_@{j}))*dummys_@{j}
@#endfor
);

//% Avg MC of Goods
exp(MC_g) = (
@#for j in 1:nsec
    +(exp(MC_@{j}))*exp(Y_@{j})*dummyg_@{j}
@#endfor
)/exp(Y_g);

//% Avg MC of Services
exp(MC_s) = (
@#for j in 1:nsec
    +(exp(MC_@{j}))*exp(Y_@{j})*dummys_@{j}
@#endfor
)/exp(Y_s);

//% Avg PL of Services
exp(PL_s) = (
@#for j in 1:nsec
    +(exp(PL_@{j}))*exp(Y_@{j})*dummys_@{j}
@#endfor
)/exp(Y_s);

//% Avg PL of goods
exp(PL_g) = (
@#for j in 1:nsec
    +(exp(PL_@{j}))*exp(Y_@{j})*dummyg_@{j}
@#endfor
)/exp(Y_g);

//% Taylor Rule
//% r = log(1/beta) + phi*pi + vi;
r = (1-rhoirule)*log(1/beta) + rhoirule*r(-1) 
+ (1-rhoirule)*phi*pi + vi;

//% Shock processes
vi = rhoi*vi(-1) + sigma_i*eps_i;

//% Goods/Services Demand Process
exp(om_g) = (1+rho_om2-rho_om1)*ombar + rho_om1*exp(om_g(-1)) - rho_om2*exp(om_g(-2)) + sigma_om*eps_om;
exp(om_s) = (1+rho_om2-rho_om1)*(1-ombar) + rho_om1*exp(om_s(-1)) - rho_om2*exp(om_s(-2)) - sigma_om*eps_om;

chi = (1-rho) + rho*chi(-1) + sigma_L_agg*epschi;

@#for i in 1:nsec
    //% TFP
    exp(A_@{i}) = (1+rho_tfp2-rho_tfp1) + rho_tfp1*exp(A_@{i}(-1)) - rho_tfp2*exp(A_@{i}(-2)) + isigma_tfp*epsA_@{i};
    
@#endfor

end;

initval;
eps_om = 0;
pi_g = 0;
pi_s = 0;
pi = 0;
chi = 1;
r = log(1/beta);
om_g=log(ombar);
om_s=log(1-ombar);
end;

steady;
ss1 = oo_.steady_state;


@#include "solution_block.mod" 
